@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2023 - Raphael Araújo e Silva <raphael@pgmodeler.io>
+# Copyright 2006-2025 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,13 @@
 */
 
 #include "modelexportform.h"
-#include "widgets/taskprogresswidget.h"
-#include "settings/configurationform.h"
 #include "guiutilsns.h"
+#include "utilsns.h"
+#include "connectionsconfigwidget.h"
+#include "pgsqlversions.h"
+#include <QThread>
 
-bool ModelExportForm::low_verbosity = false;
+bool ModelExportForm::low_verbosity {false};
 
 ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(parent, f)
 {
@@ -31,15 +33,25 @@ ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(p
 
 	sql_file_sel = new FileSelectorWidget(this);
 	sql_file_sel->setFileDialogTitle(tr("Export model to SQL file"));
+	sql_file_sel->setAcceptMode(QFileDialog::AcceptSave);
+	sql_file_sel->setAllowFilenameInput(true);
+	sql_file_sel->setFileIsMandatory(false);
+	sql_file_sel->setAppendSuffix(true);
 	export_to_file_grid->addWidget(sql_file_sel, 1, 1);
 
 	img_file_sel = new FileSelectorWidget(this);
 	img_file_sel->setFileDialogTitle(tr("Export model to graphics file"));
 	img_file_sel->setAcceptMode(QFileDialog::AcceptSave);
-	export_to_img_grid->addWidget(img_file_sel, 1, 1, 1, 2);
+	img_file_sel->setAllowFilenameInput(true);
+	img_file_sel->setFileIsMandatory(false);
+	img_file_sel->setAppendSuffix(true);
+	export_to_img_grid->addWidget(img_file_sel, 2, 1, 1, 3);
 
 	dict_file_sel = new FileSelectorWidget(this);
 	dict_file_sel->setFileDialogTitle(tr("Export model to data dictionary"));
+	dict_file_sel->setAllowFilenameInput(true);
+	dict_file_sel->setFileIsMandatory(false);
+	dict_file_sel->setAppendSuffix(true);
 	export_to_dict_grid->addWidget(dict_file_sel, 1, 1, 1, 5);
 
 	htmlitem_del=new HtmlItemDelegate(this);
@@ -53,10 +65,15 @@ ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(p
 	export_to_dict_gb->setFocusProxy(export_to_dict_rb);
 	export_to_img_gb->setFocusProxy(export_to_img_rb);
 
+	connect(sql_file_sel, &FileSelectorWidget::s_selectorChanged, this, &ModelExportForm::enableExport);
 	connect(sql_file_sel, &FileSelectorWidget::s_fileSelected, this, &ModelExportForm::enableExport);
 	connect(sql_file_sel, &FileSelectorWidget::s_selectorCleared, this, &ModelExportForm::enableExport);
+
+	connect(img_file_sel, &FileSelectorWidget::s_selectorChanged, this, &ModelExportForm::enableExport);
 	connect(img_file_sel, &FileSelectorWidget::s_fileSelected, this, &ModelExportForm::enableExport);
 	connect(img_file_sel, &FileSelectorWidget::s_selectorCleared, this, &ModelExportForm::enableExport);
+
+	connect(dict_file_sel, &FileSelectorWidget::s_selectorChanged, this, &ModelExportForm::enableExport);
 	connect(dict_file_sel, &FileSelectorWidget::s_fileSelected, this, &ModelExportForm::enableExport);
 	connect(dict_file_sel, &FileSelectorWidget::s_selectorCleared, this, &ModelExportForm::enableExport);
 
@@ -70,6 +87,16 @@ ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(p
 	connect(export_btn, &QPushButton::clicked, this, &ModelExportForm::exportModel);
 	connect(drop_chk, &QCheckBox::toggled, drop_db_rb, &QRadioButton::setEnabled);
 	connect(drop_chk, &QCheckBox::toggled, drop_objs_rb, &QRadioButton::setEnabled);
+	connect(drop_db_rb, &QCheckBox::toggled, force_db_drop_chk, &QRadioButton::setEnabled);
+
+	connect(drop_chk, &QCheckBox::toggled, this, [this](bool toggled) {
+		force_db_drop_chk->setEnabled(toggled && drop_db_rb->isChecked());
+	});
+
+	connect(drop_objs_rb, &QCheckBox::toggled, this, [this](bool toggled){
+		if(toggled)
+			force_db_drop_chk->setChecked(false);
+	});
 
 	connect(export_thread, &QThread::started, &export_hlp, [this](){
 		output_trw->setUniformRowHeights(true);
@@ -78,7 +105,7 @@ ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(p
 			export_hlp.exportToDBMS();
 		else if(export_to_img_rb->isChecked())
 		{
-			if(png_rb->isChecked())
+			if(img_fmt_cmb->currentIndex() == 0)
 				export_hlp.exportToPNG();
 			else
 				export_hlp.exportToSVG();
@@ -102,15 +129,11 @@ ModelExportForm::ModelExportForm(QWidget *parent, Qt::WindowFlags f) : QDialog(p
 	connect(cancel_btn, &QToolButton::clicked, this, &ModelExportForm::cancelExport);
 	connect(connections_cmb, &QComboBox::currentIndexChanged, this, &ModelExportForm::editConnections);
 
-	connect(svg_rb, &QRadioButton::toggled, zoom_cmb, &QComboBox::setDisabled);
-	connect(svg_rb, &QRadioButton::toggled, zoom_lbl, &QLabel::setDisabled);
-	connect(svg_rb, &QRadioButton::toggled, page_by_page_chk, &QCheckBox::setDisabled);
-	connect(svg_rb, &QRadioButton::toggled, this, &ModelExportForm::selectImageFormat);
-	connect(png_rb, &QRadioButton::toggled, this, &ModelExportForm::selectImageFormat);
+	connect(img_fmt_cmb, &QComboBox::currentIndexChanged, this, &ModelExportForm::selectImageFormat);
 
 	connect(ignore_error_codes_chk, &QCheckBox::toggled, error_codes_edt, &QLineEdit::setEnabled);
-	connect(dict_standalone_rb, &QRadioButton::toggled, this, &ModelExportForm::selectDataDictMode);
-	connect(dict_split_rb, &QRadioButton::toggled, this, &ModelExportForm::selectDataDictMode);
+	connect(dict_mode_cmb, &QComboBox::currentIndexChanged, this, &ModelExportForm::selectDataDictMode);
+	connect(dict_format_cmb, &QComboBox::currentIndexChanged, this, &ModelExportForm::selectDataDictMode);
 	connect(sql_standalone_rb, &QRadioButton::toggled, this, &ModelExportForm::selectSQLExportMode);
 	connect(sql_split_rb, &QRadioButton::toggled, this, &ModelExportForm::selectSQLExportMode);
 	connect(sql_split_rb, &QRadioButton::toggled, code_options_cmb, &QComboBox::setEnabled);
@@ -140,13 +163,39 @@ void ModelExportForm::setLowVerbosity(bool value)
 
 void ModelExportForm::exec(ModelWidget *model)
 {
-	if(model)
-	{
-		this->model=model;
-		ConnectionsConfigWidget::fillConnectionsComboBox(connections_cmb, true, Connection::OpExport);
-		selectExportMode();
-		QDialog::exec();
-	}
+	if(!model)
+		return;
+
+	this->model = model;
+	ConnectionsConfigWidget::fillConnectionsComboBox(connections_cmb, true, Connection::OpExport);
+
+	selectExportMode();
+
+#ifdef DEMO_VERSION
+	#warning "DEMO VERSION: export to DBMS is disabled in demo version."
+	export_to_dbms_rb->blockSignals(true);
+	export_to_dbms_rb->setEnabled(false);
+	export_to_dbms_gb->setEnabled(false);
+	export_to_dbms_rb->setChecked(false);
+	export_to_dbms_rb->blockSignals(false);
+
+	#warning "DEMO VERSION: export to data dictionary is disabled in demo version."
+	export_to_dict_rb->blockSignals(true);
+	export_to_dict_rb->setEnabled(false);
+	export_to_dict_gb->setEnabled(false);
+	export_to_dict_rb->blockSignals(false);
+
+	#warning "DEMO VERSION: export to data PNG limited to zoom factor of 50%."
+	zoom_cmb->setCurrentText("30%");
+	zoom_cmb->setEnabled(false);
+
+	#warning "DEMO VERSION: export to data SVG is disabled in demo version."
+	img_fmt_cmb->setEnabled(false);
+
+	export_to_file_rb->setChecked(true);
+#endif
+
+	QDialog::exec();
 }
 
 void ModelExportForm::handleErrorIgnored(QString err_code, QString err_msg, QString cmd)
@@ -156,7 +205,7 @@ void ModelExportForm::handleErrorIgnored(QString err_code, QString err_msg, QStr
 	item=GuiUtilsNs::createOutputTreeItem(output_trw, tr("Error code <strong>%1</strong> found and ignored. Proceeding with export.").arg(err_code),
 																					 QPixmap(GuiUtilsNs::getIconPath("alert")), nullptr, false);
 
-	GuiUtilsNs::createOutputTreeItem(output_trw, GuiUtilsNs::formatMessage(err_msg),
+	GuiUtilsNs::createOutputTreeItem(output_trw, UtilsNs::formatMessage(err_msg),
 																			QPixmap(GuiUtilsNs::getIconPath("alert")),	item, false, true);
 
 	GuiUtilsNs::createOutputTreeItem(output_trw, cmd, QPixmap(), item, false, true);
@@ -165,18 +214,18 @@ void ModelExportForm::handleErrorIgnored(QString err_code, QString err_msg, QStr
 void ModelExportForm::updateProgress(int progress, QString msg, ObjectType obj_type, QString cmd, bool is_code_gen)
 {
 	QTreeWidgetItem *item=nullptr;
-	QString text=GuiUtilsNs::formatMessage(msg);
+	QString text=UtilsNs::formatMessage(msg);
 	QPixmap ico;
 
 	progress_lbl->setText(text);
 	progress_pb->setValue(progress);
 
-	if(obj_type!=ObjectType::BaseObject)
-		ico=QPixmap(GuiUtilsNs::getIconPath(obj_type));
+	if(obj_type != ObjectType::BaseObject)
+		ico = QPixmap(GuiUtilsNs::getIconPath(obj_type));
 	else if(!cmd.isEmpty())
-		ico=QPixmap(GuiUtilsNs::getIconPath("sqlcode"));
+		ico = QPixmap(GuiUtilsNs::getIconPath("sqlcode"));
 	else
-		ico=QPixmap(GuiUtilsNs::getIconPath("info"));
+		ico = QPixmap(GuiUtilsNs::getIconPath("info"));
 
 	ico_lbl->setPixmap(ico);
 
@@ -194,6 +243,23 @@ void ModelExportForm::exportModel()
 {
 	try
 	{
+		// Alerting the user about dropping objects/database
+		if(export_to_dbms_rb->isChecked() && drop_chk->isChecked())
+		{
+			Messagebox msg_box;
+			QString msg;
+
+			if(drop_db_rb->isChecked())
+				msg = tr("<strong>CAUTION:</strong> You are about to drop an entire database from the chosen server! All data will be completely wiped out. Do you really want to proceed?");
+			else
+				msg = tr("<strong>CAUTION:</strong> You are about to drop objects in a database of the chosen server! Data can be lost in the process. Do you really want to proceed?");
+
+			msg_box.show(tr("Warning"), msg, Messagebox::AlertIcon, Messagebox::YesNoButtons);
+
+			if(msg_box.isRejected())
+				return;
+		}
+
 		output_trw->clear();
 		settings_tbw->setTabEnabled(1, true);
 		settings_tbw->setCurrentIndex(1);
@@ -205,11 +271,11 @@ void ModelExportForm::exportModel()
 		{
 			viewp=new QGraphicsView(model->scene);
 
-			if(png_rb->isChecked())
+			if(img_fmt_cmb->currentIndex() == 0)
 				export_hlp.setExportToPNGParams(model->scene, viewp, img_file_sel->getSelectedFile(),
 																				zoom_cmb->itemData(zoom_cmb->currentIndex()).toDouble(),
 																				show_grid_chk->isChecked(), show_delim_chk->isChecked(),
-																				page_by_page_chk->isChecked());
+																				 page_by_page_chk->isChecked(), override_bg_color_chk->isChecked());
 			else
 				export_hlp.setExportToSVGParams(model->scene, img_file_sel->getSelectedFile(),
 																				show_grid_chk->isChecked(),
@@ -231,27 +297,35 @@ void ModelExportForm::exportModel()
 				progress_lbl->setText(tr("Saving file '%1'").arg(sql_file_sel->getSelectedFile()));
 				export_hlp.setExportToSQLParams(model->db_model, sql_file_sel->getSelectedFile(),
 																				pgsqlvers_cmb->currentText(), sql_split_rb->isChecked(),
-																				static_cast<DatabaseModel::CodeGenMode>(code_options_cmb->currentIndex()));
+																				static_cast<DatabaseModel::CodeGenMode>(code_options_cmb->currentIndex()),
+																				gen_drop_file_chk->isChecked());
 				export_thread->start();
 			}
 			else if(export_to_dict_rb->isChecked())
 			{
-				export_hlp.setExportToDataDictParams(model->db_model, dict_file_sel->getSelectedFile(), incl_index_chk->isChecked(), dict_split_rb->isChecked());
+				export_hlp.setExportToDataDictParams(model->db_model, dict_file_sel->getSelectedFile(),
+																						 incl_index_chk->isChecked(),
+																						 dict_mode_cmb->currentIndex() == 1,
+																						 dict_format_cmb->currentIndex() == 1);
 				export_thread->start();
 			}
 			//Exporting directly to DBMS
 			else
 			{
+				Connection *conn=reinterpret_cast<Connection *>(connections_cmb->itemData(connections_cmb->currentIndex()).value<void *>());
 				QString version;
-				Connection *conn=reinterpret_cast<Connection *>(connections_cmb->itemData(connections_cmb->currentIndex()).value<void *>());			
 
 				//If the user chose a specific version
 				if(pgsqlvers1_cmb->isEnabled())
 					version=pgsqlvers1_cmb->currentText();
 
-				export_hlp.setExportToDBMSParams(model->db_model, conn, version, ignore_dup_chk->isChecked(),
-												 drop_chk->isChecked() && drop_db_rb->isChecked(),
-												 drop_chk->isChecked() && drop_objs_rb->isChecked());
+				export_hlp.setExportToDBMSParams(model->db_model, conn, version,
+																				 ignore_dup_chk->isChecked(),
+																				 drop_chk->isChecked() && drop_db_rb->isChecked(),
+																				 drop_chk->isChecked() && drop_objs_rb->isChecked(),
+																				 false, false,
+																				 drop_chk->isChecked() && force_db_drop_chk->isChecked(),
+																				 run_in_transaction_chk->isChecked());
 
 				if(ignore_error_codes_chk->isChecked())
 					export_hlp.setIgnoredErrors(error_codes_edt->text().simplified().split(' '));
@@ -262,10 +336,8 @@ void ModelExportForm::exportModel()
 	}
 	catch(Exception &e)
 	{
-		Messagebox msg_box;
-
 		finishExport(tr("Exporting process aborted!"));
-		msg_box.show(e);
+		Messagebox::error(e, __PRETTY_FUNCTION__, __FILE__, __LINE__);
 	}
 }
 
@@ -283,13 +355,17 @@ void ModelExportForm::selectExportMode()
 		rb->blockSignals(false);
 	}
 
+	sql_file_sel->setFileIsMandatory(export_to_file_rb->isChecked());
+	img_file_sel->setFileIsMandatory(export_to_img_rb->isChecked());
+	dict_file_sel->setFileIsMandatory(export_to_dict_rb->isChecked());
+
 	pgsqlvers1_cmb->setEnabled(export_to_dbms_rb->isChecked() && pgsqlvers_chk->isChecked());
 	enableExport();
 }
 
 void ModelExportForm::captureThreadError(Exception e)
 {
-	QTreeWidgetItem *item=GuiUtilsNs::createOutputTreeItem(output_trw, GuiUtilsNs::formatMessage(e.getErrorMessage()),
+	QTreeWidgetItem *item=GuiUtilsNs::createOutputTreeItem(output_trw, UtilsNs::formatMessage(e.getErrorMessage()),
 																														QPixmap(GuiUtilsNs::getIconPath("error")), nullptr, false, true);
 
 	GuiUtilsNs::createExceptionsTree(output_trw, e, item);
@@ -297,7 +373,7 @@ void ModelExportForm::captureThreadError(Exception e)
 	ico_lbl->setPixmap(QPixmap(GuiUtilsNs::getIconPath("error")));
 	finishExport(tr("Exporting process aborted!"));
 
-	throw Exception(e.getErrorMessage(), e.getErrorCode(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	Messagebox::error(e, __PRETTY_FUNCTION__, __FILE__, __LINE__);
 }
 
 void ModelExportForm::cancelExport()
@@ -324,6 +400,8 @@ void ModelExportForm::handleExportFinished()
 	finishExport(msg);
 	ico_lbl->setPixmap(ico);
 	GuiUtilsNs::createOutputTreeItem(output_trw, msg, ico);
+
+	qApp->alert(this);
 }
 
 void ModelExportForm::finishExport(const QString &msg)
@@ -366,18 +444,10 @@ void ModelExportForm::closeEvent(QCloseEvent *event)
 
 void ModelExportForm::editConnections()
 {
-	try
+	if(connections_cmb->currentIndex()==connections_cmb->count()-1)
 	{
-		if(connections_cmb->currentIndex()==connections_cmb->count()-1)
-		{
-			ConnectionsConfigWidget::openConnectionsConfiguration(connections_cmb, true);
-			emit s_connectionsUpdateRequest();
-		}
-	}
-	catch(Exception &e)
-	{
-		Messagebox msg_box;
-		msg_box.show(e);
+		ConnectionsConfigWidget::openConnectionsConfiguration(connections_cmb, true);
+		emit s_connectionsUpdateRequest();
 	}
 
 	enableExport();
@@ -385,39 +455,60 @@ void ModelExportForm::editConnections()
 
 void ModelExportForm::enableExport()
 {
-	export_btn->setEnabled((export_to_dbms_rb->isChecked() && connections_cmb->currentIndex() > 0 && connections_cmb->currentIndex() != connections_cmb->count()-1) ||
-												 (export_to_file_rb->isChecked() && !sql_file_sel->getSelectedFile().isEmpty()) ||
-												 (export_to_img_rb->isChecked() && !img_file_sel->getSelectedFile().isEmpty()) ||
-												 (export_to_dict_rb->isChecked() && !dict_file_sel->getSelectedFile().isEmpty()));
+	export_btn->setEnabled((export_to_dbms_rb->isChecked() && connections_cmb->currentIndex() > 0 &&
+													 connections_cmb->currentIndex() != connections_cmb->count()-1) ||
+												 (export_to_file_rb->isChecked() && !sql_file_sel->hasWarning()) ||
+												 (export_to_img_rb->isChecked() && !img_file_sel->hasWarning()) ||
+												 (export_to_dict_rb->isChecked() && !dict_file_sel->hasWarning()));
 }
 
 void ModelExportForm::selectImageFormat()
 {
-	if(png_rb->isChecked())
+	bool is_png = img_fmt_cmb->currentIndex() == 0;
+
+	if(is_png)
 	{
 		img_file_sel->setMimeTypeFilters({"image/png", "application/octet-stream"});
 		img_file_sel->setDefaultSuffix("png");
+		override_bg_color_chk->setEnabled(true);
 	}
 	else
 	{
 		img_file_sel->setMimeTypeFilters({"image/svg+xml", "application/octet-stream"});
 		img_file_sel->setDefaultSuffix("svg");
+		override_bg_color_chk->setEnabled(false);
 	}
+
+	zoom_cmb->setEnabled(is_png);
+	zoom_lbl->setEnabled(is_png);
+	page_by_page_chk->setEnabled(is_png);
 }
 
 void ModelExportForm::selectDataDictMode()
 {
-	if(dict_standalone_rb->isChecked())
+	if(dict_mode_cmb->currentIndex() == 0)
 	{
-		dict_file_sel->setMimeTypeFilters({"text/html", "application/octet-stream"});
-		dict_file_sel->setDefaultSuffix("html");
+		if(dict_format_cmb->currentIndex() == 0)
+		{
+			dict_file_sel->setMimeTypeFilters({"text/html", "application/octet-stream"});
+			dict_file_sel->setDefaultSuffix("html");
+		}
+		else
+		{
+			dict_file_sel->setMimeTypeFilters({"text/markdown", "application/octet-stream"});
+			dict_file_sel->setDefaultSuffix("md");
+		}
+
 		dict_file_sel->setAcceptMode(QFileDialog::AcceptSave);
-		dict_file_sel->setFileMode(QFileDialog::AnyFile);
+		dict_file_sel->setDirectoryMode(false);
+		dict_file_sel->setFileMustExist(false);
 	}
 	else
 	{
 		dict_file_sel->setDefaultSuffix("");
-		dict_file_sel->setFileMode(QFileDialog::Directory);
+		dict_file_sel->setMimeTypeFilters({});
+		dict_file_sel->setDirectoryMode(true);
+		dict_file_sel->setFileMustExist(false);
 		dict_file_sel->setAcceptMode(QFileDialog::AcceptOpen);
 	}
 }
@@ -428,13 +519,16 @@ void ModelExportForm::selectSQLExportMode()
 	{
 		sql_file_sel->setMimeTypeFilters({"application/sql", "application/octet-stream"});
 		sql_file_sel->setDefaultSuffix("sql");
-		sql_file_sel->setFileMode(QFileDialog::AnyFile);
+		sql_file_sel->setFileMustExist(false);
+		sql_file_sel->setDirectoryMode(false);
 		sql_file_sel->setAcceptMode(QFileDialog::AcceptSave);
 	}
 	else
 	{
+		sql_file_sel->setMimeTypeFilters({});
 		sql_file_sel->setDefaultSuffix("");
 		sql_file_sel->setAcceptMode(QFileDialog::AcceptOpen);
-		sql_file_sel->setFileMode(QFileDialog::Directory);
+		sql_file_sel->setFileMustExist(false);
+		sql_file_sel->setDirectoryMode(true);
 	}
 }
